
#include <Arduino.h>
#include "Config.h"
#include "TimeClock.h"
#include <EEPROM.h>

#include "Input.h"
#include "Player.h"

#include <Snooze.h>

extern Conf conf;
extern elapsedMillis sleepTime;

// ********************************************************************************************************************************
// *******************************   Joystick  **************************************************************************************
// ********************************************************************************************************************************

extern Joystick  joyX,joyY ;      // declaration objet joystick

Joystick::Joystick(
                uint8_t pin,                             // pin 
                uint16_t neutral,                        // position neutre entre 0 et 1023
                uint8_t deadBand,    
                int8_t direction,                         // direction -1 ou 1
                uint16_t max,
                uint16_t min
                ) : _pin(pin),  _neutral(neutral), _min(min), _max(max), _deadBand(deadBand),  _direction(direction)
  {  
    pinMode(_pin, INPUT);
  }

int16_t Joystick::get() 
  { 
    _read = analogRead(_pin);
    int16_t _value;
    
    if (_read  > _max) _read = _max;
    if (_read  < _min) _read = _min;
  
    if (_read  > _neutral + _deadBand) _value=_direction  * map (_read, _neutral + _deadBand , _max, 0, 500);  // (_value-_deadBand);
    else if (_read < _neutral -_deadBand) _value = _direction  * map (_read, _min, _neutral -_deadBand, -500, 0);           // (_value+_deadBand);
         else _value=0;
    
    return _value; 
  }

uint16_t Joystick::rawGet()   {  return _read;  }

// ********************************************************************************************************************************
// *******************************   Switch  **************************************************************************************
// ********************************************************************************************************************************

boolean  Switch::_anyShortPress = LOW;  // pour n'importe quel bouton

uint32_t Switch::_lastPress = millis();
  
Switch::Switch(uint8_t initPin) 
  : _pin(initPin)
  {
  pinMode(initPin , INPUT_PULLUP);
  _type = PIN;
  }

Switch::Switch(Joystick *joy, int16_t th) 
    : _joy(joy), _th(th)  
    {  _type = JOY;  }

void Switch::begin() { _timePress=millis(); }

#define LONG_PRESS_DELAY  1500

void Switch::handle() 
    {  
    
      // reinitialisation des états de sorti des boutons
      _shortPress = LOW;   _longPress = LOW;
     
      _currentMillis = millis();
      
      if (_type==PIN) _state=!digitalRead(_pin);
      else if (_th > 0 ) _state = (_joy->get() > _th );
           else _state = (_joy->get() < _th );
    
      if (_state && !_lastState) { // on presse le bouton
          if (conf.enableSound) tone (5,80,20);
          _lastState = _state; 
          _timePress = _currentMillis;
          sleepTime = 0;
          
          _resetLongPress = HIGH;
          _lastPress = _currentMillis;
          
          _shortPress = HIGH;
          _anyShortPress = HIGH;      // Variable static pour savoir si n'importe quel bouton a été pressé
          }
      
      if (!_state && _lastState) // on relache le bouton
          { 
          _lastState = _state; 
          _resetLongPress = LOW;
          _timePress = _currentMillis;
          }
      
      if (_resetLongPress && _currentMillis-_timePress > LONG_PRESS_DELAY ) 
        {
            if (conf.enableSound) tone (5,120,40);
            _longPress = HIGH;
            _resetLongPress = LOW;
        } 
    }

boolean Switch::state()  { return _state; }

boolean Switch::shortPress() {  return _shortPress;  }
       
boolean Switch::longPress() {  return _longPress;  }

boolean Switch::anyShortPress()  { return _anyShortPress;  }

void Switch::resetAnyShortPress()   { _anyShortPress = LOW; }

// ********************************************************************************************************************************
// *******************************   Gyro **************************************************************************************
// ********************************************************************************************************************************

// https://www.nxp.com/docs/en/application-note/AN3461.pdf
// https://www.i2cdevlib.com/docs/html/class_m_p_u6050.html

#define MPU6050_ADDRESS    0x69  // 
#define SIGNAL_PATH_RESET  0x68
#define I2C_SLV0_ADDR      0x37
#define ACCEL_CONFIG       0x1C 
#define MOT_THR            0x1F  // Motion detection threshold bits [7:0]
#define MOT_DUR            0x20  // Duration counter threshold for motion interrupt generation, 1 kHz rate, LSB = 1 ms
#define MOT_DETECT_CTRL    0x69
#define INT_ENABLE         0x38
#define WHO_AM_I_MPU6050   0x75 // Should return 0x68
#define INT_STATUS 0x3A

#define MOTION_THRESHOLD 1
#define MOTION_EVENT_DURATION 40

#define SCALE_FOR_MAP   1.0
  
  void Gyro::begin ()

{
    Wire.begin();

    Wire.setClock(400000UL); // Set I2C frequency to 400kHz
    
    initialize();

    setAccelerometerPowerOnDelay(3);
    
    setInterruptMode(true); // Interrupts enabled
    
    //setInterruptLatch(0); // Interrupt pulses when triggered instead of remaining on until cleared
    
    setIntMotionEnabled(true); // Interrupts sent when motion detected
    
    setMotionDetectionThreshold(MOTION_THRESHOLD);
    setMotionDetectionDuration(MOTION_EVENT_DURATION);

    // Set sensor filter mode.
  // 0 -> Reset (disable high pass filter)
  // 1 -> On (5Hz)
  // 2 -> On (2.5Hz)
  // 3 -> On (1.25Hz)
  // 4 -> On (0.63Hz)
  // 5 -> Hold (Future outputs are relative to last output when this mode was set)
   
    setDHPFMode(1);

    setDMPEnabled(true);

    getMotion9(&_ax, &_ay, &_az, &_gx, &_gy, &_gz, &_mx, &_my, &_mz);
    _roll =  atan2(_ax, _az ) * RAD_TO_DEG; // 
    _pitch = atan(-_ay / sqrt(_ax * _ax + _az * _az) ) * RAD_TO_DEG;

    kalmanX.setAngle(_roll);
    kalmanY.setAngle(_pitch);
    
}
  
  void Gyro::handle()
        {
         
         double dt = (double)(micros() - timer) / 1000000; // Calculate delta time
         timer = micros();
         
         getMotion9(&_ax, &_ay, &_az, &_gx, &_gy, &_gz,  &_mx, &_my, &_mz);

         _gxRate = _gx / 131.0; // Convert to deg/s  
         _gyRate = _gy / 131.0; // Convert to deg/s
                       
         if (abs(_gx) > 500 ) sleepTime = 0 ;
         
            //  OFFSET
            _gx += 130;
            
        _roll = atan(_ay / sqrt(_ax * _ax + _az * _az) ) * RAD_TO_DEG;
        _pitch = atan2( -_ax, _az ) * RAD_TO_DEG; 
        
        if ((_pitch < -90 && _kalPitch > 90) || (_pitch > 90 && _kalPitch < -90)) 
                  {
                    kalmanY.setAngle(_pitch);
                    _kalPitch = _pitch;
                    _gyroPitch = _pitch;
                  } 
          else   _kalPitch = kalmanY.getAngle(_pitch, _gyRate, dt);
            
          if (abs(_kalPitch) > 90)  _gxRate = -_gxRate; // Invert rate, so it fits the restriced accelerometer reading
          
          _kalRoll = kalmanX.getAngle(_roll, _gxRate, dt); // Calculate the angle using a Kalman filter

          _gyroRoll  += _gxRate * dt; // Calculate gyro angle without any filter
          _gyroPitch += _gyRate * dt;
          
          //Serial.print(_kalRoll); Serial.print("  "); Serial.println(_kalPitch);
         }
         
double Gyro::getRoll()
        {
           return _kalRoll ;
        }

double Gyro::getPitch()
        {
           return _kalPitch ;
        }

#define PITCH_STEP      10

Vector Gyro::Joystick()
    {
        Vector r;
        int16_t joyx = (int16_t) _kalRoll;
        //int16_t joyy = (int16_t) _kalPitch;
        
        int8_t sx = joyx / abs ( joyx) ; 
        //int8_t sy = joyy / abs ( joyy) ; 

        if ( joyx < 3 && joyx > - 3 ) r.x = 0;
        else if ( joyx < 5 && joyx > - 5  ) r.x = sx;
             else if ( joyx < 7 && joyx > - 7  ) r.x = 2 * sx;
                  else if ( joyx < 9 && joyx > -9  ) r.x = 3 * sx;
                       else r.x = 4 * sx;

        if ( _pitch > _pitchDelay + PITCH_STEP ) 
          {
            _pitchDelay = _pitch  ;
            r.y = -1;
          }
        else if ( _pitch < _pitchDelay - PITCH_STEP) 
          {
            _pitchDelay = _pitch ;
            r.y = 1;
          }
             else r.y = 0;

        //Serial.print("roll : " ); Serial.print(_roll ); Serial.print("      pitch : " ); Serial.println(_pitch );
        
        return r;
    }

int16_t Gyro::ax()
        {
           return _ax ;
        }

int16_t Gyro::ay()
        {
           return _ay;
        }


        
//#endif
